/* 
 * Copyright (c) 2000 Mike Muir
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	Parts of this code derived from examples by Takanori Watanabe
 *	Makefile.in and configure.in based on Chris D. Faulhaber's wmlmmon files.
 * 
 * Device operations for wmhm.c
 * 
 * $Id: operations.c,v 1.5 2001/07/22 05:00:55 mmuir Exp $
 *
 */

#include "config.h"

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <machine/cpufunc.h>
#include <machine/speaker.h>

#ifdef HAVE_SMBUS
#include <machine/smb.h>
#endif
#include "machine/speaker.h"

#include "operations.h"

tone_t tone_array[40];

static int io_file;
static int open_smb;
extern int SMBUS;

int 
PlaySpeaker(char *tune_array)
{
    int i;
    int open_spkr;
    if ((open_spkr=open("/dev/speaker", O_WRONLY)) < 0) {
        fprintf(stderr, "Failed to open /dev/speaker for writing.\n"
                        "No Permission?\n"
        );
        return(1);
    }
    for (i = 0; i < 30; ++i) {
        tone_array[i].frequency = 3000+pow(i,2);
        tone_array[i].duration = 1;
    }
    for (i = 30; i < 40; ++i) {
        tone_array[i].frequency = 3000+pow(30,2);
        tone_array[i].duration = 2;
    }

    tone_array[40].frequency = 1000;
    tone_array[40].duration = 0;

    if (ioctl(open_spkr, SPKRTUNE, tone_array) == -1) {
        perror("ioctl");
        exit(-1);
    }

    close(open_spkr);

    return(0);
}

int 
OpenIO()
{
#ifdef HAVE_SMBUS
    if (SMBUS) {
        u_char smb_return;
        struct smbcmd cmd;
        cmd.slave=0x5a;
        cmd.data.byte_ptr=&smb_return;
        if ((open_smb=open("/dev/smb0", 000)) < 0) {
            fprintf(stderr, "Failed to open /dev/smb0.\nPossible reasons:\n"
                            "- Your kernel does not support SMBUS.\n"
                            "- You are not running wmhm suid root.\n"
                            "- WMHM is already running.\n"
            );
            exit(1);
        }
        cmd.cmd=0x47;
        if( ioctl(open_smb, SMB_READB, &cmd) == -1) {
			perror("ioctl");
			return -1;
        }
        return(0);
    } 
    else {
        if ((io_file=open("/dev/io", 000)) < 0) {
            fprintf(stderr, "Failed to open /dev/io.\n"
                            "Possible reasons:\n"
                            "- You are not running wmhm suid root.\n"
                            "- WMHM is already running\n");
	  		exit (-1);
		}
        return(0);
    }
#else
    if((io_file=open("/dev/io",000))<0) {
        fprintf(stderr, "Failed to open /dev/io.\n");
        exit (1);
    }
    return(0);
#endif
}

int ReadByte(u_char *return_value, int addr)
{
#ifdef HAVE_SMBUS
    if (SMBUS) {
        u_char smb_return;
        struct smbcmd cmd;
        cmd.slave=0x5a;
        cmd.data.byte_ptr=&smb_return;
        cmd.cmd=addr;
        if(ioctl(open_smb,SMB_READB,&cmd)==-1) {
            perror("IOCTL");
            return(-1);
        }
        *return_value = smb_return;
        return(0);
    } 
    else {
        outb(WBIO1,addr);
        *return_value = inb(WBIO2);
        return(0);
    }
#else 
    outb(WBIO1,addr);
    *return_value = inb(WBIO2);
    return(0);
#endif
}

int WriteByte(int addr, int value)
{
#ifdef HAVE_SMBUS
    if (SMBUS) {
        u_char smb_return;
        struct smbcmd cmd;
        cmd.slave=0x5a;
        cmd.data.byte_ptr=&smb_return;
        cmd.cmd=addr;
        cmd.data.byte=value;
        if(ioctl(open_smb,SMB_WRITEB,&cmd)==-1) {
            perror("IOCTL");
            return(-1);
        }
    } 
    else {
        outb(WBIO1,addr);
        outb(WBIO2,value);
    }
#else 
    outb(WBIO1,addr);
    outb(WBIO2,value);
#endif
    return(0);
}

